﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Microsoft.Practices.EnterpriseLibrary.Caching;
using System.Diagnostics;
using Microsoft.Practices.EnterpriseLibrary.Caching.Expirations;

namespace StackOverflowFlair.Extensions
{
    public static class CacheControllerExtensions
    {
        private static ICacheManager dataCache = CacheFactory.GetCacheManager("ControllerCache");

        public static T CacheRead<T>(this Controller controller, string key, Func<T> refreshFunction)
        {
            object obj = dataCache[key];
            if (obj != null && obj is T)
            {
                Trace.WriteLine("Cache Hit: " + key);
                return (T)obj;
            }
            else
            {
                Trace.WriteLine("Cache Miss: " + key);
                T newObj = refreshFunction.Invoke();
                dataCache.Add(key, newObj, CacheItemPriority.Normal, null, new AbsoluteTime(new TimeSpan(0, 10, 0)), new SlidingTime(new TimeSpan(0, 2, 0)));
                return newObj;
            }
        }
    }
}
