﻿namespace StackOverflowFlair.Extensions
{
    using System;
    using System.Collections.Generic;
    using System.IO.Compression;
    using System.Linq;
    using System.Web;
    using System.Web.Mvc;

    public class CompressFilter : ActionFilterAttribute
    {
        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            HttpRequestBase request = filterContext.HttpContext.Request;

            string[] supported = new string[] { "gzip", "deflate" };

            IEnumerable<string> preferredOrder = new AcceptList(request.Headers["Accept-Encoding"], supported);

            string preferred = preferredOrder.FirstOrDefault();

            HttpResponseBase response = filterContext.HttpContext.Response;

            switch (preferred)
            {
                case "gzip":
                    response.AppendHeader("Content-Encoding", "gzip");
                    response.Filter = new GZipStream(response.Filter, CompressionMode.Compress);
                    break;

                case "deflate":
                    response.AppendHeader("Content-Encoding", "deflate");
                    response.Filter = new DeflateStream(response.Filter, CompressionMode.Compress);
                    break;

                case "identity":
                default:
                    break;
            }
        }
    }
}